package org.openxma.xmadsl.diagnostics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.openxma.xmadsl.util.Pair;
import org.openxma.xmadsl.util.Tuples;

/**
 * @author Sebastian Zarnekow - Initial contribution and API
 */
public class ListBasedDiagnosticConsumer implements IDiagnosticConsumer {

	private final List<Diagnostic> diagnostics;
	
	private final Set<Pair<Integer, Integer>> coveredNodes;

	private boolean diagnosticsConsumed;
	
	public ListBasedDiagnosticConsumer() {
		this.diagnostics = new ArrayList<Diagnostic>();
		this.coveredNodes = new HashSet<Pair<Integer,Integer>>();
		this.diagnosticsConsumed = false;
	}
	
	public void consume(Diagnostic diagnostic) {
		final Pair<Integer, Integer> newRange = Tuples.create(diagnostic.getOffset(), diagnostic.getLength());
		if (coveredNodes.add(newRange)) {
			boolean changed = this.diagnostics.add(diagnostic);
			diagnosticsConsumed |= changed;
		}
	}

	public boolean hasConsumedDiagnostics() {
		return diagnosticsConsumed;
	}
	
	public List<Diagnostic> getResult() {
		return Collections.unmodifiableList(diagnostics);
	}
	
}
