package org.openxma.xmadsl.formatting;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.openxma.xmadsl.model.ComplexElement;
import org.openxma.xmadsl.model.Component;
import org.openxma.xmadsl.model.ComponentModel;
import org.openxma.xmadsl.model.DataMapping;
import org.openxma.xmadsl.model.DataObjectVariable;
import org.openxma.xmadsl.model.Page;
import org.openxma.xmadsl.model.ReferencedXMAComposite;
import org.openxma.xmadsl.model.ReferencedXMAPage;

public class ComponentModelFormatting extends ModelFormatting {
	
	private static ComponentModelFormatting instance;	
	public static ModelFormatting getInstance() {
		if (instance == null) {
			instance = new ComponentModelFormatting();
		}
		return instance;
	}		
	
	@Override
	public ArrayList<String> formatModel(EObject obj) {
		
		if (obj == null || obj instanceof ComponentModel == false) {
			throw new IllegalArgumentException("Element to be formatted has to be an instance of ComponentModel: "+obj);
		}
		
		ComponentModel componentModel = (ComponentModel)obj;
		
        ArrayList<String> data = new ArrayList<String>();		
        
        formatImports(componentModel.getImports(),data);                
        data.add("");
        formatReferencedXMAComponent(componentModel,data);
        data.add("");
        return data;
	}
	
	protected void formatReferencedXMAPage(ReferencedXMAPage refPage, ArrayList<String> output) {
		
		 // Begin of page
  	 output.add("");		
		 output.add(indent(1) + "Page xma::" + refPage.getXmaPage().getNamClass() + " {");
		 
		 // DataObjectVariables
       List<DataObjectVariable> dovList = refPage.getDataobjects();
   	 output.add("");
       for (int i=0;i<dovList.size();i++) {
       	DataObjectVariable bd = (DataObjectVariable)dovList.get(i);
          String entityName = bd.getName();	            	
       	output.add(indent(2) + "Data "+bd.getType().getName()+" "+entityName);
       }
       
       // Datamapping
       if (refPage.getDataMapping() != null) {
	         output.add("");
	         output.add(indent(2)+"DataMapping {");
	         List<DataMapping> mappingList = refPage.getDataMapping().getMapping();
	         for (DataMapping mapping: mappingList) {
	        	 String xmaName = mapping.getControl().getNamInstance();
	        	 String dataObjectVariableName = mapping.getField().getObject().getName();
	        	 String attributName = mapping.getField().getAttribute().getName();        	 
	        	 output.add(indent(3)+"xma::"+xmaName+" <-> "+dataObjectVariableName+"."+attributName);    	         	 
	         }
	         output.add(indent(2)+"}");
       }
       
       // Composites
       if (refPage.getContent() != null) {
	         for (ComplexElement complexElement: refPage.getContent().getContainerItems()) {
	        	 if (complexElement instanceof ReferencedXMAComposite) {
	        		 ReferencedXMAComposite refComposite = (ReferencedXMAComposite)complexElement;
	        	       output.add(indent(2) + "Composite xma::" + refComposite.getXmaComposite().getNamInstance() + " {");
	                   output.add("");
	                   output.add(indent(2) + "}");
	                   output.add("");        		         		 
	        	 }
	         }
       }
            
       // End of page
	     output.add(indent(1)+"}");  		
	}
	
    /**
     * adds the collected references to the given string list
     * @param output
     */
    protected void formatReferencedXMAComponent (ComponentModel componentModel, ArrayList<String> output ) {
        if (componentModel == null || componentModel.getComponent() == null) {
            return;
        }
        Component component = componentModel.getComponent();
        output.add("Component xma::" + component.getXmaComponent().getNamClass() + " {");
//        if (dataObjectVariable.size()>0) {
//        	output.add("");
//            for (DataObjectVariable bd : dataObjectVariable) {
//                String entityName = getEntityNameForBd(bd);	            	
//            	output.add(indent(1) + "Data "+entityName+" "+bd.getName()+";");
//            }
//        }	            
//        output.add("");
        
        for (Page page : component.getPages())
        	if (page instanceof ReferencedXMAPage) {
        		ReferencedXMAPage refPage = (ReferencedXMAPage)page;
        		formatReferencedXMAPage(refPage,output);      		 
        	}
                    
        output.add("}");
    }

}
