/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.xmadsl.parser;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.openarchitectureware.emf.EcoreUtil2;
import org.openarchitectureware.workflow.WorkflowContext;
import org.openarchitectureware.workflow.WorkflowInterruptedException;
import org.openarchitectureware.workflow.issues.Issues;
import org.openarchitectureware.workflow.monitor.ProgressMonitor;
import org.openarchitectureware.xtext.parser.ErrorMsg;
import org.openarchitectureware.xtext.parser.impl.AbstractParserComponent;
import org.openarchitectureware.xtext.registry.CachingModelLoad;
import org.openarchitectureware.xtext.registry.XtextResourceSet;
import org.openarchitectureware.xtext.resource.IXtextResource;
import org.openxma.xmadsl.linking.lazy.LazyLinkingResource;
import org.openxma.xmadsl.resource.ClasspathResourceFactory;
import org.openxma.xmadsl.resource.CustomizedxmadslResourceFactory;

/**
 * Customized parser compoent to register additional emf resource factories.
 * 
 * @author <a href="mailto:michael.clay@s-itsolutions.at">Michael Clay</a>
 * @version $Revision$ $Date$
 * 
 */
public class CustomizedParserComponent extends AbstractParserComponent {
	
	private static XtextResourceSet xtextResourceSet;
	private String outputSlot = WorkflowContext.DEFAULT_SLOT;
	
	static {
		CustomizedxmadslResourceFactory.register();
		ClasspathResourceFactory.register();
	}

	protected String getFileExtension() {
		return "xmadsl";
	}
	
	@Override
	protected void invokeInternal(WorkflowContext ctx, ProgressMonitor monitor, Issues issues) {
		try {
			URI uri = EcoreUtil2.getURI(modelFile);
			XtextResourceSet xtextResourceSet = getXtextResourceSet();
//			xtextResourceSet.getLoadOptions().put(LazyLinkingResource.OPTION_RESOLVE_ALL, Boolean.TRUE);
			Resource r = CachingModelLoad.loadResource(uri, xtextResourceSet, false, true);
			if (!(r instanceof IXtextResource))
				throw new WorkflowInterruptedException("Resource is not an instance of 'IXtextResource'.");
			for (ErrorMsg err : ((IXtextResource) r).getParser().getParseErrors()) {
				issues.addError(err.getMsg() + " on line " + err.getLine() + " in " + modelFile);
			}
			if (!r.getContents().isEmpty()) {
				ctx.set(this.outputSlot, r.getContents().get(0));
			}
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static XtextResourceSet getXtextResourceSet() {
		return null==xtextResourceSet ? new XtextResourceSet() : xtextResourceSet;
	}

	public static void setXtextResourceSet(XtextResourceSet xtextResourceSet) {
		CustomizedParserComponent.xtextResourceSet = xtextResourceSet;
	}
	

	/**
	 * Sets the output slot.
	 * 
	 * @param outputSlot
	 *            the output slot
	 */
	public void setOutputSlot(String outputSlot) {
		this.outputSlot = outputSlot;
	}

	public String getOutputSlot() {
		return outputSlot;
	};
	
	
	
	
}
