/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.xmadsl.resource;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl;
import org.openarchitectureware.xtext.resource.IXtextResourceFactory;

/**
 * Represents an implementation of emf's {@link ResourceFactoryImpl} to create
 * and register 'java' aware (emf) resources.
 * 
 * @author <a href="mailto:michael.clay@s-itsolutions.at">Michael Clay</a>
 * @version $Revision$ $Date$
 * @see ClasspathResourceImpl
 */
public class ClasspathResourceFactory extends ResourceFactoryImpl implements IXtextResourceFactory {
	// some constants
	public static final String RESOURCE_EXTENSION = "java";

	public static final String CLASSPATH_RESOURCE_PREFIX = "classpath:";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl#createResource
	 * (org.eclipse.emf.common.util.URI)
	 */
	@Override
	public Resource createResource(URI uri) {
		return new ClasspathResourceImpl(uri);
	}

	/**
	 * Register this <code>ClasspathResourceFactory</code> as factory for all
	 * 'java' resources.
	 */
	public static void register() {
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(RESOURCE_EXTENSION,
				new ClasspathResourceFactory());
	}
}