/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.xmadsl.resource;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
import org.openarchitectureware.workflow.util.ResourceLoaderFactory;
import org.openxma.xmadsl.MetaModelRegistration;

/**
 * Represents an implementation of emf's resource interface to support the
 * integration of existing platform or legacy types from within a xmadsl
 * (application)model.
 * 
 * 
 * @author <a href="mailto:michael.clay@s-itsolutions.at">Michael Clay</a>
 * @version $Revision$ $Date$
 */
public class ClasspathResourceImpl extends ResourceImpl {

	private String qualifiedClassName;
	private String packageName;

	public ClasspathResourceImpl(URI uri) {
		super(uri);
		String uriAsString = uri.toString();
		this.qualifiedClassName = uriAsString.substring(11,
				uriAsString.indexOf(".java")).replaceAll("/", "\\.");
		this.packageName = this.qualifiedClassName.substring(0,
				this.qualifiedClassName.lastIndexOf("."));
	}

	@Override
	protected void doLoad(InputStream inputStream, Map<?, ?> options)
			throws IOException {

		try

		{
			Class<?> loadedClass = ResourceLoaderFactory.createResourceLoader().loadClass(this.qualifiedClassName);
			EFactory factoryInstance = MetaModelRegistration.getEPackage()
					.getEFactoryInstance();

			EClass packageClass = (EClass) MetaModelRegistration.getEPackage()
					.getEClassifier("TypeModel");
			EAttribute packageAttribute = (EAttribute) packageClass
					.getEStructuralFeature("package");
			EReference contentsStructuralFeature = (EReference) packageClass
					.getEStructuralFeature("contents");
			EObject packageObjectInstance = factoryInstance
					.create(packageClass);
			packageObjectInstance.eSet(packageAttribute, this.packageName);

			EClass valueObjectClass = (EClass) MetaModelRegistration
					.getEPackage().getEClassifier("ValueObject");
			EReference attributesStructuralFeature = (EReference) valueObjectClass
					.getEStructuralFeature("attributes");
			EAttribute nameAttribute = (EAttribute) valueObjectClass
					.getEStructuralFeature("name");
			EObject beanObjectInstance = factoryInstance
					.create(valueObjectClass);
			beanObjectInstance.eSet(nameAttribute, loadedClass.getSimpleName());

			EClass dataTypeAndTypeParameterClass = (EClass) MetaModelRegistration
					.getEPackage().getEClassifier("DataTypeAndTypeParameter");
			EClass simpleTypeClass = (EClass) MetaModelRegistration
					.getEPackage().getEClassifier("SimpleType");
			EClass typeDefinitionClass = (EClass) MetaModelRegistration
					.getEPackage().getEClassifier("TypeDefinition");

			BeanInfo beanInfo = Introspector.getBeanInfo(loadedClass);

			PropertyDescriptor[] propertyDescriptors = beanInfo
					.getPropertyDescriptors();

			for (int i = 0; i < propertyDescriptors.length; i++) {

				PropertyDescriptor propertyDescriptor = propertyDescriptors[i];

				if (null != propertyDescriptor.getWriteMethod()) {
					EClass attributeClass = (EClass) MetaModelRegistration
							.getEPackage().getEClassifier("Attribute");
					EAttribute attributeNameAttribute = (EAttribute) attributeClass
							.getEStructuralFeature("name");
					EReference attributeTypeAttribute = (EReference) attributeClass
							.getEStructuralFeature("type");

					EObject attributeInstance = factoryInstance
							.create(attributeClass);
					attributeInstance.eSet(attributeNameAttribute,
							propertyDescriptor.getName());
					// simpleType
					EObject simpleTypeInstance = factoryInstance
							.create(simpleTypeClass);
					EAttribute simpleTypeNameAttribute = (EAttribute) simpleTypeClass
							.getEStructuralFeature("name");
					EReference typeDefinitionAttribute = (EReference) simpleTypeClass
							.getEStructuralFeature("typeDefinition");
					simpleTypeInstance.eSet(simpleTypeNameAttribute,
							propertyDescriptor.getPropertyType()
									.getSimpleName());
					// simpleType.typeDefinition
					EObject typeDefinitionInstance = factoryInstance
							.create(typeDefinitionClass);
					EAttribute nativeJavaTypeAttribute = (EAttribute) typeDefinitionClass
							.getEStructuralFeature("javatype");
					typeDefinitionInstance.eSet(nativeJavaTypeAttribute,
							propertyDescriptor.getPropertyType().getName());

					simpleTypeInstance.eSet(typeDefinitionAttribute,
							simpleTypeInstance);

					// attribute.type (is DataTypeAndTypeParameter)
					EObject dataTypeAndTypeParameterInstance = factoryInstance
							.create(dataTypeAndTypeParameterClass);
					EReference dataTypeAttribute = (EReference) dataTypeAndTypeParameterClass
							.getEStructuralFeature("dataType");
					dataTypeAndTypeParameterInstance.eSet(dataTypeAttribute,
							typeDefinitionInstance);

					attributeInstance.eSet(attributeTypeAttribute,
							dataTypeAndTypeParameterInstance);
					((List) beanObjectInstance
							.eGet(attributesStructuralFeature))
							.add(attributeInstance);
				}
			}

			((List) packageObjectInstance.eGet(contentsStructuralFeature))
					.add(beanObjectInstance);

			getContents().add(packageObjectInstance);
		} catch (Throwable classNotFoundException) {
			classNotFoundException.printStackTrace();
			throw new IOException("Class with name '" + qualifiedClassName
					+ "' could not be found on classpath.");
		}
	}
}
