/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.xmadsl.resource;

import java.net.URL;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.openarchitectureware.workflow.util.ResourceLoaderFactory;

/**
 * Represents an implementation of emf's {@link URIConverter URI converter} to
 * handle the loading of 'classpath' resources.
 * 
 * @author <a href="mailto:michael.clay@s-itsolutions.at">Michael Clay</a>
 * @version $Revision$ $Date$
 */
public class ClasspathURIConverter extends ExtensibleURIConverterImpl {

	// some constants
	private static final String CLASS_FILE_EXTENSION = ".class";

	public ClasspathURIConverter() {
		getURIHandlers().add(0, new ClasspathURIHandler());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl#normalize
	 * (org.eclipse.emf.common.util.URI)
	 */
	@Override
	public URI normalize(URI uri) {

		if (uri.toString().startsWith(
				ClasspathResourceFactory.CLASSPATH_RESOURCE_PREFIX)) {

			String uriPath = uri.path();

			if (uri.hasAbsolutePath()) {
				uriPath = uriPath.substring(1);
			}

			URL url = ResourceLoaderFactory
					.createResourceLoader()
					.getResource(
							uriPath.replaceAll(
											"\\."+ ClasspathResourceFactory.RESOURCE_EXTENSION,
											CLASS_FILE_EXTENSION));
			return "file".equalsIgnoreCase(url.getProtocol()) ? URI
					.createFileURI(URI.decode(url.getFile())) : URI
					.createURI(url.toString());

		}
		// else business-as-usual
		return super.normalize(uri);
	}

}