package org.openxma.xmadsl.resource;

import java.io.InputStream;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl;
import org.openarchitectureware.xtext.parser.IXtextParser;
import org.openarchitectureware.xtext.resource.IXtextResourceFactory;
import org.openxma.xmadsl.linking.lazy.LazyLinkingResource;
import org.openxma.xmadsl.parser.XtextParser;

/**
 * Represents a specialized version of <code>ResourceFactoryImpl</code> to
 * preserve the emf URI from which the resource was created from.
 * 
 * @see org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl
 */
public class CustomizedxmadslResourceFactory extends ResourceFactoryImpl
		implements IXtextResourceFactory {
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl#createResource(org.eclipse.emf.common.util.URI)
	 */
	@Override
	public Resource createResource(URI uri) {
		return new LazyLinkingResource(uri) {
			@Override
			protected IXtextParser createParser(InputStream inputStream) {
				XtextParser xtextParser = new XtextParser(inputStream);
				xtextParser.setUri(uri);
				return xtextParser;
			}
		};
	}
	/*
	 * Register this <code>ResourceFactoryImpl</code> for <i>xmdsl</i> resources.
	 */
	public static void register() {
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				"xmadsl", new CustomizedxmadslResourceFactory());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				"dmodel", new CustomizedxmadslResourceFactory());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				"dml", new CustomizedxmadslResourceFactory());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				"pml", new CustomizedxmadslResourceFactory());
	}
}
