package org.openxma.xmadsl.scoping.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openarchitectureware.emf.EcoreUtil2;
import org.openxma.xmadsl.GenXmaDslScopeProvider;
import org.openxma.xmadsl.XmadslModelExtension;
import org.openxma.xmadsl.linking.impl.SimpleAttributeResolver;
import org.openxma.xmadsl.model.AttributeHolder;
import org.openxma.xmadsl.model.AttributeWithProperties;
import org.openxma.xmadsl.model.Component;
import org.openxma.xmadsl.model.CompositeContent;
import org.openxma.xmadsl.model.CustomizationOfGuiElement;
import org.openxma.xmadsl.model.CustomizeableField;
import org.openxma.xmadsl.model.DataView;
import org.openxma.xmadsl.model.DataViewSource;
import org.openxma.xmadsl.model.FieldReference;
import org.openxma.xmadsl.model.IField;
import org.openxma.xmadsl.model.IncludeExcludeDefinition;
import org.openxma.xmadsl.model.Page;
import org.openxma.xmadsl.model.PageContent;
import org.openxma.xmadsl.model.ParameterValue;
import org.openxma.xmadsl.model.ReferencedXMAComposite;
import org.openxma.xmadsl.model.ReferencedXMAPage;
import org.openxma.xmadsl.model.Table;
import org.openxma.xmadsl.model.TableColumn;
import org.openxma.xmadsl.model.Validator;
import org.openxma.xmadsl.model.ValidatorReference;
import org.openxma.xmadsl.model.XmadslPackage;
import org.openxma.xmadsl.scoping.IScope;

import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.HiddenWidget;
import at.spardat.xma.guidesign.NotebookPage;
import at.spardat.xma.guidesign.XMACombo;
import at.spardat.xma.guidesign.XMACompProperty;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMATable;
import at.spardat.xma.guidesign.XMATableColumn;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.XMAFunction;

import com.google.common.base.Function;
/**
 * Represents an XmaDsl specific implementation of the {@link AbstractDeclarativeScopeProvider}
 */
public class XmaDslScopeProvider extends GenXmaDslScopeProvider {
	
	public XmaDslScopeProvider() {
		setGenericFallBack(new DefaultScopeProvider());
	}
	
	public IScope scope_AttributeWithProperties_attributeHolder(AttributeWithProperties attributeWithProperties,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		IncludeExcludeDefinition includeExcludeDefinition = (IncludeExcludeDefinition) attributeWithProperties.eContainer();
		DataView dataView = (DataView) includeExcludeDefinition.eContainer();
		DataViewSource dataViewSource = dataView.getSourceDefinition().getSources().iterator().next();
		List<? extends EObject> allAttributeHolders =  XmadslModelExtension.allAttributeHolders(dataViewSource.getSource());
		return new SimpleScope(ScopedElements.allInCollection(allAttributeHolders, eClass, SimpleAttributeResolver.NAME_RESOLVER));
	}
	
	public IScope scope_ParameterValue_referencedParameter(ParameterValue parameterValue,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		if (parameterValue.eContainer() instanceof ValidatorReference) {
			Validator validator = ((ValidatorReference)parameterValue.eContainer()).getValidator();
			return new SimpleScope(ScopedElements.allInEObject(validator, eClass, SimpleAttributeResolver.NAME_RESOLVER));
		}
		return new DefaultScope(parameterValue.eResource(),eClass,new ImportUriResolver());
	}

	
	public IScope scope_IField_attributeHolder(IField field,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		List<? extends EObject> allAttributeHolders = XmadslModelExtension.allAttributeHolders(field.getObject().getType());
		return new SimpleScope(ScopedElements.allInCollection(allAttributeHolders,eClass, new Function<EObject, String>(){
			public String apply(EObject eObject) {
				if (eObject instanceof AttributeWithProperties) {
					AttributeWithProperties attributeWithProperties = (AttributeWithProperties)eObject;
					return attributeWithProperties.getAttributeHolder().getName();
				} 
				return ((AttributeHolder)eObject).getName();
			}}));
	}
	
	public IScope scope_IField_object(IField field,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		Page containingPage = getContainingPage(field);
		Component containgComponent = containingPage != null  ? getContainingComponent(containingPage) : getContainingComponent(field);
		Collection<EObject> nsDataObjectVariable = getNsDataObjectVariable(containgComponent,containingPage);
		SimpleScope simpleScope = new SimpleScope(ScopedElements.allInCollection(nsDataObjectVariable,
				eClass, SimpleAttributeResolver.NAME_RESOLVER));
		return simpleScope;
	}
	
	public IScope scope_TableColumn_attributeHolder(TableColumn tableColumn,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		List<? extends EObject> allAttributeHolders = XmadslModelExtension.allAttributeHolders(tableColumn.getObject().getType());
		return new SimpleScope(ScopedElements.allInCollection(allAttributeHolders,eClass, NAME_RESOLVER));
	}
	
	public IScope scope_Table_key(Table table,EReference eReference) {
		EClass eClass = (EClass) eReference.getEType();
		List<? extends EObject> allAttributeHolders = XmadslModelExtension.allAttributeHolders(table.getObject().getType());
		return new SimpleScope(ScopedElements.allInCollection(allAttributeHolders,eClass, NAME_RESOLVER));
	}
	
	private Page getContainingPage(EObject eObject) {
		if (eObject instanceof Page) {
			return (Page) eObject;
		} else if (eObject instanceof CustomizationOfGuiElement) {
			return ((CustomizationOfGuiElement)eObject).getPage();
		} 
		return eObject.eContainer()!=null ? getContainingPage(eObject.eContainer()) : null;
	}
	
	private Component getContainingComponent(EObject eObject) {
		if (null==eObject || eObject instanceof Component) {
			return (Component) eObject;
		} else if (eObject instanceof CustomizationOfGuiElement) {
			getContainingComponent(((CustomizationOfGuiElement)eObject).getPage());
		} 
		return getContainingComponent(eObject.eContainer());
	} 
	
	private Collection<EObject> getNsDataObjectVariable(Component component, Page page ) {
		List<EObject> elements = getNsElementsInRange(component,page);
		return EcoreUtil.getObjectsByType(elements, XmadslPackage.eINSTANCE.getDataObjectVariable());
	} 
	
	private List<EObject> getNsElementsInRange(EObject topElement,final EObject bottomElement) {
		List<EObject> nsElements = new ArrayList<EObject>(topElement.eContents());
		List<EObject> elementsWithChildrenInNs = new ArrayList<EObject>();
		for (EObject eObject : nsElements) {
			if (isNsRangeBorder(eObject)==false || containsChildrenInRange(eObject,bottomElement)) {
				elementsWithChildrenInNs.add(eObject);
			}
		}
		for (EObject object : elementsWithChildrenInNs) {
			List<EObject> nsElementsInRange = getNsElementsInRange(object,bottomElement);
			nsElements.addAll(nsElementsInRange);
		}
		return nsElements;
	}
		
	private boolean containsChildrenInRange(EObject element, EObject bottomElement) {
		return bottomElement != null ? 
				element == bottomElement || EcoreUtil2.allContents(element).contains(bottomElement)
				: false;  	
	}
	
	public boolean isNsRangeBorder(EObject eObject) {
		if (eObject instanceof Page) {
			return true;
		} else if (eObject instanceof Component) {
			return true;
		}  else if (eObject instanceof XMAPage) {
			return true;
		}  else if (eObject instanceof XMAComponent) {
			return true;
		} 
		return false;
	}
	
	protected Function<EObject, String> getNameResolver(EObject eObject) {
	   return NAME_RESOLVER;
	}
	
	public static final  IdAttributeResolver NAME_RESOLVER = new IdAttributeResolver();
    
	// TODO: refactor this if-else-if chain
	public static class IdAttributeResolver extends SimpleAttributeResolver<EObject,String> {
        public IdAttributeResolver() {
            super(String.class, "name");
        }
        public String apply(EObject eObject) {
            if (eObject instanceof XMAComponent) {
                return ((XMAComponent)eObject).getNamClass();
            } else if (eObject instanceof XMAComposite) {
                return ((XMAComposite)eObject).getNamInstance();
            } else if (eObject instanceof XMAComposite) {
                return ((XMAComposite)eObject).getNamInstance();
            } else if (eObject instanceof Component) {
                return ((Component)eObject).getXmaComponent()==null ? null : this.apply(((Component)eObject).getXmaComponent());
            } else if (eObject instanceof ReferencedXMAComposite) {
                return ((ReferencedXMAComposite)eObject).getXmaComposite()==null ? null : this.apply(((ReferencedXMAComposite)eObject).getXmaComposite());
            } else if (eObject instanceof ReferencedXMAPage) {
                return ((ReferencedXMAPage)eObject).getXmaPage()==null ? null : this.apply(((ReferencedXMAPage)eObject).getXmaPage());
            } else if (eObject instanceof XMAText) {
                return ((XMAText)eObject).getNamInstance();
            } else if (eObject instanceof HiddenWidget) {
                return ((HiddenWidget)eObject).getNamInstance();
            } else if (eObject instanceof XMATable) {
                return ((XMATable)eObject).getNamInstance();
            } else if (eObject instanceof XMACombo) {
                return ((XMACombo)eObject).getNamInstance();
            } else if (eObject instanceof XMAWidget) {
                return ((XMAWidget)eObject).getNamInstance();
            } else if (eObject instanceof XMATableColumn) {
                return ((XMATableColumn)eObject).getNamInstance();
            } else if (eObject instanceof XMAPage) {
                return ((XMAPage)eObject).getNamClass();
            } else if (eObject instanceof NotebookPage) {
                return ((NotebookPage)eObject).getNamClass();
            } else if (eObject instanceof XMACompProperty) {
                return ((XMACompProperty)eObject).getNamProperty();
            } else if (eObject instanceof AttributeWithProperties) {
                AttributeWithProperties attributeWithProperties = (AttributeWithProperties)eObject;
                return attributeWithProperties.getAttributeHolder().getName();
            } else if (eObject instanceof AttributeHolder) {
                return ((AttributeHolder)eObject).getName();
            } else if (eObject instanceof IField) {
                return ((IField)eObject).getObject().getName() + "." + XmadslModelExtension.getAttributeName((IField) eObject);
            } else if (eObject instanceof FieldReference) {
                return ((FieldReference)eObject).getObject().getName() + "." + XmadslModelExtension.getAttributeName((FieldReference) eObject);
            } else if (eObject instanceof CustomizeableField) {
                return ((CustomizeableField)eObject).getObject().getName() + "." + XmadslModelExtension.getAttributeName((CustomizeableField) eObject);
            } else if (eObject instanceof XMAFunction) {
                return ((XMAFunction)eObject).getNamFunction();
            } else if (eObject instanceof BDAttribute) {
                return ((BDAttribute)eObject).getBdObject().getNamInstance() + "." + ((BDAttribute)eObject).getNamAttrVal();
            } else if (eObject instanceof CompositeContent) {
                return this.apply(eObject.eContainer())+"-compositecontent";
            }  else if (eObject instanceof PageContent) {
                return this.apply(eObject.eContainer())+"-pagecontent";
            } 
            return super.apply(eObject);
        }
    }
	
}
