package org.openxma.xmadsl.validation;

import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EValidator;

import com.google.inject.Inject;

/**
 * @author Sebastian Zarnekow - Initial contribution and API
 */
public abstract class AbstractInjectableValidator implements EValidator {

	@Inject
	public void register(EValidatorRegistrar registrar) {
		List<? extends EPackage> packages = getEPackages();
		for (EPackage ePackage : packages) {
			registrar.register(ePackage, this);
		}
	}

	protected List<? extends EPackage> getEPackages() {
		throw new UnsupportedOperationException("please overwrite");
	}

	public boolean validate(EDataType eDataType, Object value, DiagnosticChain diagnostics, Map<Object, Object> context) {
		return true;
	}

	public boolean validate(EObject eObject, DiagnosticChain diagnostics, Map<Object, Object> context) {
		return validate(eObject.eClass(), eObject, diagnostics, context);
	}

}